# frozen_string_literal: true

module Hermes
  # Base serializer
  class ApplicationSerializer < ActiveModel::Serializer
    require Rails.root
                 .join(*%w[lib core_extentions mongoid huntdown_object_id_class])
                 .to_s
    # declare responsible keys at serializer begining
    def initialize(*params)
      super
      @responsible_displaying_keys = %i[id first_name last_name]
    end

    def responsible_displaying_keys(*keys)
      @responsible_displaying_keys.concat(keys.to_a)
    end

    attributes :id, :responsible, :readers, :writers, :read_level
    
    # default cache of 1.day
    cache key: proc { "#{object.cache_key}" }, expires_in: 1.day
    
    # Override initial cache_key value
    def object_cache_key
      added_to = added_to_object_cache_key.blank? ? '' : '-' + added_to_object_cache_key.to_s
      options = @instance_options.except(:prefixes, :template, :layout, :namespace, :scope, :scope_name, :per_page, :page)

      "#{self.class.name}##{object.cache_key}#{added_to}?instance_options=#{options.map{|k,v| "#{k}=#{v}"}.join('&')}"
    end

    # method to add value to cache key
    def added_to_object_cache_key
      ""
    end 

    def id
      object&.id&.to_s
    end

    attributes(
      if @options&.fetch('is_test', false)
        :test_created_at
      else
        :created_at
      end
    )

    attributes :updated_at

    def user_val(key)
      object&.user&.send(key)
    end

    def display_to_db_key(key)
      case key
      when :id
        :_id
      else
        key
      end
    end

    def formated_user_val(key)
      formated_key = display_to_db_key(key)
      huntdown_object_id_class(user_val(formated_key))
    end

    def format_responsible
      return unless object&.user

      @responsible_displaying_keys.each_with_object({}) do |key, h|
        h[key] = formated_user_val(key)
      end
    end

    def responsible
      format_responsible
    end

    def current_user
      scope
    end

    def writable?
      return false unless scope

      object&.writable_by?(scope)
    end

    def writers
      writable? ? object&.writers : nil
    end

    def readable?
      return false unless scope

      try(:is_writable?) || object&.readable_by?(scope)
    end

    def readers
      readable? ? object&.readers : nil
    end

    def read_level
      if writable?
        'write'
      elsif readable?
        'read'
      else
        'guest'
      end
    end
  end
end
