require 'rest-client'
require 'base64'
require 'nokogiri'

module Hermes
  class BoClient::Client
    class ClientError < StandardError
      attr_reader :response
      
      def initialize(response)
        super(message)
        @response = response
        if false
          begin
            message = "#{error_message} - CODE '#{code}'"
          rescue => e
            message = "Error while call API, Unparsable response (error: #{e.inspect}) : #{self.response.to_s.force_encoding("UTF-8")}"
          end
        end
        message = "Error : #{response.to_s.force_encoding("UTF-8")}"
        
        super(message)
      end
      
      def code
        to_xml&.at_xpath('code')&.text
      end
      
      def error_message
        to_xml&.at_xpath('message')&.text
      end
      
      def to_xml
        Nokogiri::XML(response.to_s).at_xpath('//errorResponse')
      end
      
      def request
        response.request
      end
    end
    
    class Forbidden < StandardError;
    end
    class NotFound < StandardError;
    end
    
    attr_accessor :base_url
    attr_accessor :user
    attr_accessor :password
    attr_accessor :logger
    
    def initialize(attrs = {})
      @base_url = attrs.fetch(:base_url, BoClient.config.base_url)
      @user = attrs.fetch(:user, BoClient.config.user)
      @password = attrs.fetch(:password, BoClient.config.password)
      @logger = attrs.fetch(:logger, BoClient.config.logger)
    end

    def endpoint_for(path)
      base_resource[path]
    end

    def general_call(path, payload , params, classname, object_id, method)
      begin
        return unless instance = classname.constantize.find(object_id)

        endpoint = endpoint_for(path)

        Hermes::LogJob.add_to_queue(level: 'info', code: "BO_CLIENT_#{method}", action: "#{classname} : #{object_id} -- #{method} #{endpoint.url}", data: { params: params }.as_json.transform_values{|v| v.to_s})
        if @logger
          @logger.debug "#{method} #{endpoint.url}"
          @logger.debug "Params :#{params}"
          @logger.debug "Payload : #{payload}, "
        end

        response = case method
          when 'GET'
            endpoint.get(params)
          when 'POST'
            endpoint.post(payload, params)
          when 'PUT'
            endpoint.put(payload, params)
          end

        instance.set(last_send_has_error: false, updated_at: DateTime.current) if instance.last_send_has_error

        Hermes::LogJob.add_to_queue(level: 'info', code: "BO_CLIENT_#{method}_SUCCESS", action: "#{classname} : #{object_id} -- #{method} #{endpoint.url}", data: { response: response }.as_json.transform_values{|v| v.to_s})
        @logger.debug "#{method} #{endpoint.url} SUCCESS" if @logger

        response
      rescue => e
        if e.respond_to?(:response)
          data = e.response.to_s.force_encoding('UTF-8')
          code = e.response&.code
        else 
          data = e.inspect
          code = nil 
        end 
        
        instance.add_error( "#{e.class.name} ----> #{data}", path, payload, code)
        
        Hermes::LogJob.add_to_queue(level: 'error', code: "BO_CLIENT_#{method}_ERROR", action: "#{classname} : #{object_id} -- Error", data: data.as_json)
        @logger.debug "#{method} #{endpoint.url} Error : #{data}" if @logger

        false
      end
    end 
    
    def get(path, classname, object_id, payload, params = {})
      general_call(path, nil , params, classname, object_id, 'GET')
    end
    
    def post(path, classname, object_id, payload, params = {})
      general_call(path, payload, params, classname, object_id, 'POST')
    end

    def put(path, classname, object_id, payload, params = {})
      general_call(path, payload, params, classname, object_id, 'PUT')
    end

    def base_resource(reload = false)
      if !@base_resource || reload
        auth_header = 'Basic ' + Base64.encode64("#{@user}:#{@password}").chomp
        @base_resource = RestClient::Resource.new(
          @base_url,
          headers: {authorization: auth_header, 'Content-Type' => 'application/xml'},
          log: logger,
          verify_ssl: false
        )
      end
      @base_resource
    end
  end
end
