require 'rails_helper'
require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

describe Hermes::Api::Dmo::NotificationCenter::SubscriptionsController, :type => :api do
  
  routes {Hermes::Engine.routes}
  include Devise::Test::ControllerHelpers
  let!(:user) { create(:user) }
  let!(:applicant) { create(:applicant, notifications_count: 5) }
  let!(:new_key) { 'testing.key' }

  describe 'Actions' do
    before :each do
      authenticated_header(request, user)
    end

    it 'return subscription' do
      get :show, targetable_type: 'Hermes::Applicant', id: applicant.id, subscription_id: applicant.subscription.id

      expect(response).to have_http_status(200)
      expect(json['data']['id']).to eq(applicant.subscription.id.to_s)
    end

    it 'should subscribe to a new key' do
      put :subscribe, targetable_type: 'Hermes::Applicant', id: applicant.id, subscription_id: applicant.subscription.id, key: new_key

      expect(response).to have_http_status(200)
      expect(applicant.subscription.subscribed_to_email?(new_key)).to be(true)
    end

    it 'should unsubscribe to an existing key' do
      applicant.subscription.subscribe_to_email(new_key)
      put :unsubscribe, targetable_type: 'Hermes::Applicant', id: applicant.id, subscription_id: applicant.subscription.id, key: new_key

      expect(response).to have_http_status(200)
      expect(applicant.subscription.subscribed_to_email?(new_key)).to be(false)
    end

    it 'should return error if provider does not exist' do
      put :unsubscribe, targetable_type: 'Hermes::Applicant', id: applicant.id, subscription_id: applicant.subscription.id, key: new_key, provider: :hello_word

      expect(response).to have_http_status(404)
    end

    it 'serializer cache cleared when unsubscribe/subscribe' do 
      applicant.subscription.subscribe_to_email(new_key)
      get :show, targetable_type: 'Hermes::Applicant', id: applicant.id, subscription_id: applicant.subscription.id
      expect(json['data']['attributes']['keys'].last['email']).to eq(true)

      put :unsubscribe, targetable_type: 'Hermes::Applicant', id: applicant.id, subscription_id: applicant.subscription.id, key: new_key
      expect(json['data']['attributes']['keys'].last['email']).to eq(false)

      put :subscribe, targetable_type: 'Hermes::Applicant', id: applicant.id, subscription_id: applicant.subscription.id, key: new_key
      expect(json['data']['attributes']['keys'].last['email']).to eq(true)
    end 

  end

  describe 'update_keys' do 
    before :each do
      authenticated_header(request, user)
      user.subscription.set(
        keys: [
          {name: "message.new", email: true, web: true, slack: false, sms: false},
          {name: "user.new_reserved_offer", email: false, web: false, slack: false, sms: false},
          {name: "user.new_published_offer", email: true, web: false, slack: false, sms: false},
        ]
      )
    end

    it 'not found' do 
      put :update_keys, targetable_type: 'User', id: user.id, subscription_id: "not found"
      expect(response).to have_http_status(404)
    end 

    it 'missing params' do 
      put :update_keys, targetable_type: 'User', id: user.id, subscription_id: user.subscription.id
      expect(response).to have_http_status(422)
    end 

    it 'wrong_subscribed_params' do 
      put :update_keys, targetable_type: 'User', id: user.id, subscription_id: user.subscription.id, keys: [
        {"key"=>"user.new_reserved_offer", "provider"=>"email", subscribed: 1},
        {"key"=>"user.new_published_offer", subscribed: true},
      ]
      expect(response).to have_http_status(422)
      expect(json['errors'].first['title']).to match(/wrong_subscribed_params/)
    end 

    it 'wrong_provider_params' do 
      put :update_keys, targetable_type: 'User', id: user.id, subscription_id: user.subscription.id, keys: [
        {"key"=>"user.new_reserved_offer", "provider"=>"lol", subscribed: true},
        {"key"=>"user.new_published_offer", subscribed: false},
      ]
      expect(response).to have_http_status(422)
      expect(json['errors'].first['title']).to match(/wrong_provider_params/)
    end 

    it 'wrong_key_params' do 
      put :update_keys, targetable_type: 'User', id: user.id, subscription_id: user.subscription.id, keys: [
        {"key"=>"lol", "provider"=>"email", subscribed: true},
        {"key"=>"user.new_published_offer", subscribed: false},
      ]
      expect(response).to have_http_status(422)
      expect(json['errors'].first['title']).to match(/wrong_key_params/)
    end 

    it 'works' do 
      put :update_keys, targetable_type: 'User', id: user.id, subscription_id: user.subscription.id, keys: [
        {"key"=>"user.new_reserved_offer", "provider"=>"email", subscribed: true},
        {"key"=>"user.new_published_offer", subscribed: false},
      ]
      expect(response).to have_http_status(200)
      keys = user.subscription.reload.keys
      expect(keys.detect{ |k| k['name'] == 'user.new_reserved_offer'}['email']).to eq(true)
      expect(keys.detect{ |k| k['name'] == 'user.new_published_offer'}['email']).to eq(false)
      attributes_keys = json['data']['attributes']['keys']
      expect(attributes_keys[1]['name']).to eq('user.new_reserved_offer')
      expect(attributes_keys[1]['email']).to eq(true)
      expect(attributes_keys[2]['name']).to eq('user.new_published_offer')
      expect(attributes_keys[2]['email']).to eq(false)
    end 
    
  end

  describe 'check indexes' do 
    it 'works' do 
      NotificationCenter::Subscription.create_indexes
      indexes_names = NotificationCenter::Subscription.collection.indexes.to_a.map{ |i| i[:name]}
      expect(indexes_names).to include('subscriber_id_1')
      expect(indexes_names).to include('subscriber_id_1')
      expect(indexes_names).to include('subscriber_type_1_subscriber_id_1')
    end 
  end 
end